package Loove 
{
	import flash.accessibility.Accessibility;
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.World;
	import net.flashpunk.FP;
	
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	
	import net.flashpunk.Sfx;
	
	import net.flashpunk.graphics.Text;
	
	public class DuetWorld extends World
	{
		public static const OBST_TYPE_ENUM_MAX:int = 10;
		
		private const MAX_NUM_OBSTACLES:Number = 18;
		
		public var mPlayer0:Player = new Player(false);
		public var mPlayer1:Player = new Player(true);
		private var mTimeTillNextTile:Number = 3;
		private var mNumObstacles:int = 0;
		
		private var mGameState:int = 0;
		private var mTimeSinceStateChange:Number = 0;
		private var mSplashScreen:SplashScreen = new SplashScreen();
		private var mInstruction:Instruction = new Instruction();
		private var mFadeScreen:FadeScreen = new FadeScreen();
		private var mScoreScreen:ScoreScreen = new ScoreScreen();
		private var mVignette:Vignette = new Vignette();
		private var mScoreEntity:ScoreEntity = new ScoreEntity();
		private var mHitFlash:HitFlash = new HitFlash();
		
		private var mObstacle0Vector:Vector.<Obstacle> = new Vector.<Obstacle>();
		private var mObstacle1Vector:Vector.<Obstacle> = new Vector.<Obstacle>();
		private var mObstacle2Vector:Vector.<Obstacle> = new Vector.<Obstacle>();
		private var mObstacle3Vector:Vector.<Obstacle> = new Vector.<Obstacle>();
		private var mObstacle4Vector:Vector.<Obstacle> = new Vector.<Obstacle>();
		private var mObstacle5Vector:Vector.<Obstacle> = new Vector.<Obstacle>();
		private var mPoints:Number = 0;
		
		[Embed(source = '../../assets/intro.mp3')] private const INTRO:Class;
		[Embed(source = '../../assets/LooveLoop.mp3')] private const MUSICLOOP:Class;
		[Embed(source = '../../assets/outro.mp3')] private const OUTRO:Class;
		
		private var mIntroSound:Sfx = new Sfx(INTRO);
		private var mMusicLoop:Sfx = new Sfx(MUSICLOOP);
		private var mOutroSound:Sfx = new Sfx(OUTRO);
		
		//private static var obstCount:int = 8;
					
		public function DuetWorld() 
		{
			//FP.console.enable();
			//FP.console.toggleKey = Key.F5;
			
			mIntroSound.play(0.4);
			
			add(new Background());
			
			for (var i:int = 0; i < MAX_NUM_OBSTACLES; ++i)
			{
				var newObst0:Obstacle = new Obstacle(0);
				mObstacle0Vector.push(newObst0);
				var newObst1:Obstacle = new Obstacle(1);
				mObstacle1Vector.push(newObst1);
				var newObst2:Obstacle = new Obstacle(2);
				mObstacle2Vector.push(newObst2);
				var newObst3:Obstacle = new Obstacle(3);
				mObstacle3Vector.push(newObst3);
				var newObst4:Obstacle = new Obstacle(4);
				mObstacle4Vector.push(newObst4);
				var newObst5:Obstacle = new Obstacle(5);
				mObstacle5Vector.push(newObst5);
			}
			
			add(mFadeScreen);
			mFadeScreen.fadeIn();
			add(mSplashScreen);
			
			FP.randomizeSeed();
		}
		
		private function startGame():void
		{
			for (var i:int = 0; i < MAX_NUM_OBSTACLES; ++i)
			{
				var newObst0:Obstacle = mObstacle0Vector[i];
				newObst0.x = 180;
				newObst0.y = 700;
				add(newObst0);
				var newObst1:Obstacle = mObstacle1Vector[i];
				newObst1.x = 180;
				newObst1.y = 700;
				add(newObst1);
				var newObst2:Obstacle = mObstacle2Vector[i];
				newObst2.x = 180;
				newObst2.y = 700;
				add(newObst2);
				var newObst3:Obstacle = mObstacle3Vector[i];
				newObst3.x = 180;
				newObst3.y = 700;
				add(newObst3);
				var newObst4:Obstacle = mObstacle4Vector[i];
				newObst4.x = 180;
				newObst4.y = 700;
				add(newObst4);
				var newObst5:Obstacle = mObstacle5Vector[i];
				newObst5.x = 180;
				newObst5.y = 700;
				add(newObst5);
			}
			
			add(mVignette);
			
			add(mPlayer0);
			add(mPlayer1);
			
			if (!mMusicLoop.playing)
			{
				mMusicLoop.loop(0.5);
			}
		}
		
		override public function update():void
		{
			mTimeSinceStateChange += FP.elapsed;
			// splash fade out
			if (mGameState == 0)
			{
				if (mTimeSinceStateChange > 4 || Input.pressed(Key.SPACE))
				{
					trace("fade out splash");
					mGameState = 1;
					mTimeSinceStateChange = 0;
					mFadeScreen.fadeOut();
				}
			}
			// splash to instruction, fade in
			else if (mGameState == 1)
			{
				if (mTimeSinceStateChange > 1)
				{
					trace("fade in instruction");
					add(mInstruction);
					remove(mSplashScreen);
					mFadeScreen.fadeIn();
					mGameState = 2;
				}
			}
			// instruction fade out
			else if (mGameState == 2)
			{
				if (Input.pressed(Key.SPACE))
				{
					trace("instuction fade out");
					startGame();
					mInstruction.fadeOut();
					mGameState = 3;
				}
			}
			// instruction to game
			else if (mGameState == 3)
			{
				if (mInstruction.completed)
				{
					trace("start game");
					remove(mInstruction);
					mGameState = 4;
				}
			}
			// game update
			else if (mGameState == 4)
			{
				mTimeTillNextTile -= FP.elapsed;
				if (mTimeTillNextTile < 0)
				{
					//var randomType:Number = FP.random * (OBST_TYPE_ENUM_MAX - 1);
					//var nextObstType:int = Math.round(randomType);
					var nextObstType:int = FP.rand(OBST_TYPE_ENUM_MAX);
					trace("spawn obst: ", nextObstType);
					//spawnObstacle(obstCount);
					//++obstCount;
					spawnObstacle(nextObstType);
				}
			}
			// game over fading in
			else if (mGameState == 5)
			{
				if (mFadeScreen.completed)
				{
					for (var i:int = 0; i < MAX_NUM_OBSTACLES; ++i)
					{
						var newObst0:Obstacle = mObstacle0Vector[i];
						remove(newObst0);
						var newObst1:Obstacle = mObstacle1Vector[i];
						remove(newObst1);
						var newObst2:Obstacle = mObstacle2Vector[i];
						remove(newObst2);
						var newObst3:Obstacle = mObstacle3Vector[i];
						remove(newObst3);
						var newObst4:Obstacle = mObstacle4Vector[i];
						remove(newObst4);
						var newObst5:Obstacle = mObstacle5Vector[i];
						remove(newObst5);
					}
					remove(mVignette);
					remove(mPlayer0);
					remove(mPlayer1);
			
					mFadeScreen.fadeIn();
					mScoreScreen.fadeIn();
					add(mScoreScreen);
					add(mScoreEntity);
					mScoreEntity.updateText(mPoints.toString());
					mGameState = 6;
				}
			}
			// game over to instruction or fade out
			else if (mGameState == 6)
			{
				if (Input.pressed(Key.ESCAPE))
				{
					trace("game over to instruction");
					mInstruction.fadeIn();
					add(mInstruction);
					remove(mScoreScreen);
					remove(mScoreEntity);
					mGameState = 2;
				}
				if (Input.pressed(Key.SPACE))
				{
					trace("game over to game");
					mScoreScreen.fadeOut();
					remove(mScoreEntity);
					startGame();
					mGameState = 7;
				}
			}
			// game over to game
			else if (mGameState == 7)
			{
				if (mScoreScreen.completed)
				{
					trace("restart game");
					remove(mScoreScreen);
					mGameState = 4;
				}
			}
			else
			{
				trace("invalid game state");
			}
			
			super.update();
		}
		
		private function spawnObstacle(obstType:int):void
		{
			// 0 = triangle
			// 1 = rect hor
			// 2 = rect vert
			// 3 = square
			// 4 = hourglass
			// 5 = hourglass vert
			
			// 2 at outside
			if (obstType == 0)
			{
				var obst0:Obstacle = getNextInactive0();
				if (obst0)
				{
					obst0.x = 15;
					obst0.y = -100;
				}
				var obst1:Obstacle = getNextInactive2();
				if (obst1)
				{
					obst1.x = 290;
					obst1.y = -100;
				}
				mTimeTillNextTile = 5;
			}
			// stairs
			else if (obstType == 1)
			{
				var obst2:Obstacle = getNextInactive3();
				if (obst2)
				{
					obst2.x = 130;
					obst2.y = -180;
				}
				var obst3:Obstacle = getNextInactive3();
				if (obst3)
				{
					obst3.x = 200;
					obst3.y = -100;
				}
				/*var obst4:Obstacle = getNextInactive3();
				if (obst4)
				{
					obst4.x = 220;
					obst4.y = -100;
				}*/
				mTimeTillNextTile = 5;
			}
			// bar
			else if (obstType == 2)
			{
				var obst5:Obstacle = getNextInactive1();
				if (obst5)
				{
					obst5.x = 105;
					obst5.y = -100;
				}
				var obst13:Obstacle = getNextInactive1();
				if (obst13)
				{
					obst13.x = 195;
					obst13.y = -100;
				}
				mTimeTillNextTile = 4;
			}
			// tunnel
			else if (obstType == 3)
			{
				var obst6:Obstacle = getNextInactive0();
				if (obst6)
				{
					obst6.x = 30;
					obst6.y = -100;
				}
				var obst7:Obstacle = getNextInactive0();
				if (obst7)
				{
					obst7.x = 320;
					obst7.y = -100;
				}
				var obst8:Obstacle = getNextInactive0();
				if (obst8)
				{
					obst8.x = 30;
					obst8.y = -170;
				}
				var obst9:Obstacle = getNextInactive0();
				if (obst9)
				{
					obst9.x = 320;
					obst9.y = -170;
				}
				var obst10:Obstacle = getNextInactive0();
				if (obst10)
				{
					obst10.x = 30;
					obst10.y = -240;
				}
				var obst11:Obstacle = getNextInactive0();
				if (obst11)
				{
					obst11.x = 320;
					obst11.y = -240;
				}
				mTimeTillNextTile = 7;
			}
			// a
			else if (obstType == 4)
			{
				var obsta0:Obstacle = getNextInactive0();
				if (obsta0)
				{
					obsta0.x = 150;
					obsta0.y = -100;
				}
				var obsta1:Obstacle = getNextInactive2();
				if (obsta1)
				{
					obsta1.x = 100;
					obsta1.y = -300;
				}
				var obsta2:Obstacle = getNextInactive1();
				if (obsta2)
				{
					obsta2.x = 200;
					obsta2.y = -300;
				}
				var obsta3:Obstacle = getNextInactive3();
				if (obsta3)
				{
					obsta3.x = 170;
					obsta3.y = -500;
				}
				var obsta4:Obstacle = getNextInactive3();
				if (obsta4)
				{
					obsta4.x = 240;
					obsta4.y = -570;
				}
				mTimeTillNextTile = 17;
			}
			// b
			else if (obstType == 5)
			{
				var obstb0:Obstacle = getNextInactive5();
				if (obstb0)
				{
					obstb0.x = 150;
					obstb0.y = -100;
				}
				var obstb1:Obstacle = getNextInactive2();
				if (obstb1)
				{
					obstb1.x = 110;
					obstb1.y = -300;
				}
				var obstb2:Obstacle = getNextInactive1();
				if (obstb2)
				{
					obstb2.x = 150;
					obstb2.y = -400;
				}
				var obstb3:Obstacle = getNextInactive1();
				if (obstb3)
				{
					obstb3.x = 220;
					obstb3.y = -435;
				}
				var obstb4:Obstacle = getNextInactive3();
				if (obstb4)
				{
					obstb4.x = 100;
					obstb4.y = -600;
				}
				mTimeTillNextTile = 17;
			}
			// c
			else if (obstType == 6)
			{
				var obstc0:Obstacle = getNextInactive1();
				if (obstc0)
				{
					obstc0.x = 70;
					obstc0.y = -100;
				}
				var obstc1:Obstacle = getNextInactive1();
				if (obstc1)
				{
					obstc1.x = 230;
					obstc1.y = -100;
				}
				var obstc2:Obstacle = getNextInactive2();
				if (obstc2)
				{
					obstc2.x = 100;
					obstc2.y = -300;
				}
				var obstc3:Obstacle = getNextInactive2();
				if (obstc3)
				{
					obstc3.x = 230;
					obstc3.y = -300;
				}
				var obstc4:Obstacle = getNextInactive0();
				if (obstc4)
				{
					obstc4.x = 15;
					obstc4.y = -450;
				}
				var obstc5:Obstacle = getNextInactive0();
				if (obstc5)
				{
					obstc5.x = 170;
					obstc5.y = -450;
				}
				var obstc6:Obstacle = getNextInactive0();
				if (obstc6)
				{
					obstc6.x = 325;
					obstc6.y = -450;
				}
				mTimeTillNextTile = 17;
			}
			// d
			else if (obstType == 7)
			{
				var obstd0:Obstacle = getNextInactive0();
				if (obstd0)
				{
					obstd0.x = 30;
					obstd0.y = -100;
				}
				var obstd1:Obstacle = getNextInactive0();
				if (obstd1)
				{
					obstd1.x = 300;
					obstd1.y = -100;
				}
				var obstd2:Obstacle = getNextInactive2();
				if (obstd2)
				{
					obstd2.x = 175;
					obstd2.y = -250;
				}
				var obstd3:Obstacle = getNextInactive1();
				if (obstd3)
				{
					obstd3.x = 140;
					obstd3.y = -350;
				}
				var obstd4:Obstacle = getNextInactive1();
				if (obstd4)
				{
					obstd4.x = 210;
					obstd4.y = -400;
				}
				var obstd5:Obstacle = getNextInactive5();
				if (obstd5)
				{
					obstd5.x = 100;
					obstd5.y = -550;
				}
				mTimeTillNextTile = 17;
			}
			// e
			else if (obstType == 8)
			{
				var obste0:Obstacle = getNextInactive3();
				if (obste0)
				{
					obste0.x = 185;
					obste0.y = -100;
				}
				var obste1:Obstacle = getNextInactive3();
				if (obste1)
				{
					obste1.x = 115;
					obste1.y = -170;
				}
				var obste2:Obstacle = getNextInactive2();
				if (obste2)
				{
					obste2.x = 25;
					obste2.y = -300;
				}
				var obste3:Obstacle = getNextInactive2();
				if (obste3)
				{
					obste3.x = 300;
					obste3.y = -300;
				}
				var obste4:Obstacle = getNextInactive0();
				if (obste4)
				{
					obste4.x = 168;
					obste4.y = -350;
				}
				var obste5:Obstacle = getNextInactive0();
				if (obste5)
				{
					obste5.x = 130;
					obste5.y = -420;
				}
				var obste6:Obstacle = getNextInactive0();
				if (obste6)
				{
					obste6.x = 320;
					obste6.y = -500;
				}
				var obste7:Obstacle = getNextInactive1();
				if (obste7)
				{
					obste7.x = 40;
					obste7.y = -600;
				}
				mTimeTillNextTile = 18;
			}
			// f
			else if (obstType == 9)
			{
				var obstf0:Obstacle = getNextInactive3();
				if (obstf0)
				{
					obstf0.x = 110;
					obstf0.y = -100;
				}
				var obstf1:Obstacle = getNextInactive3();
				if (obstf1)
				{
					obstf1.x = 195;
					obstf1.y = -100;
				}
				var obstf2:Obstacle = getNextInactive2();
				if (obstf2)
				{
					obstf2.x = 170;
					obstf2.y = -250;
				}
				var obstf3:Obstacle = getNextInactive0();
				if (obstf3)
				{
					obstf3.x = 35;
					obstf3.y = -300;
				}
				var obstf4:Obstacle = getNextInactive0();
				if (obstf4)
				{
					obstf4.x = 290;
					obstf4.y = -300;
				}
				var obstf5:Obstacle = getNextInactive1();
				if (obstf5)
				{
					obstf5.x = 185;
					obstf5.y = -430;
				}
				var obstf6:Obstacle = getNextInactive1();
				if (obstf6)
				{
					obstf6.x = 115;
					obstf6.y = -430;
				}
				var obstf7:Obstacle = getNextInactive1();
				if (obstf7)
				{
					obstf7.x = 60;
					obstf7.y = -620;
				}
				var obstf8:Obstacle = getNextInactive1();
				if (obstf8)
				{
					obstf8.x = 235;
					obstf8.y = -620;
				}
				mTimeTillNextTile = 19;
			}
		}
		
		private function getNextInactive0():Obstacle
		{
			for (var i:int = 0; i < mObstacle0Vector.length; ++i)
			{
				var obst:Obstacle = mObstacle0Vector[i];
				if (obst.y > 600)
				{
					return obst;
				}
			}
			return null;
		}
		
		private function getNextInactive1():Obstacle
		{
			for (var i:int = 0; i < mObstacle1Vector.length; ++i)
			{
				var obst:Obstacle = mObstacle1Vector[i];
				if (obst.y > 600)
				{
					return obst;
				}
			}
			return null;
		}
		
		private function getNextInactive2():Obstacle
		{
			for (var i:int = 0; i < mObstacle2Vector.length; ++i)
			{
				var obst:Obstacle = mObstacle2Vector[i];
				if (obst.y > 600)
				{
					return obst;
				}
			}
			return null;
		}
		
		private function getNextInactive3():Obstacle
		{
			for (var i:int = 0; i < mObstacle3Vector.length; ++i)
			{
				var obst:Obstacle = mObstacle3Vector[i];
				if (obst.y > 600)
				{
					return obst;
				}
			}
			return null;
		}
		
		private function getNextInactive4():Obstacle
		{
			for (var i:int = 0; i < mObstacle4Vector.length; ++i)
			{
				var obst:Obstacle = mObstacle4Vector[i];
				if (obst.y > 600)
				{
					return obst;
				}
			}
			return null;
		}
		
		private function getNextInactive5():Obstacle
		{
			for (var i:int = 0; i < mObstacle5Vector.length; ++i)
			{
				var obst:Obstacle = mObstacle5Vector[i];
				if (obst.y > 600)
				{
					return obst;
				}
			}
			return null;
		}
		
		public function setLevelEndScreen(points:Number, collPoint:Point):void
		{
			if (mGameState == 4)
			{
				trace("setLevelEndScreen");
				mPoints = points;
				mFadeScreen.fadeOut();
				mOutroSound.play(0.5);
				mGameState = 5;
				
				if (collPoint)
				{
					mHitFlash.x = collPoint.x - FP.halfWidth + mPlayer0.halfWidth;
					mHitFlash.y = collPoint.y - FP.halfHeight + mPlayer0.halfHeight;
					add(mHitFlash);
				}
			}
		}
	}
}